// bits that would be in input.js but are currently failing with astring

var A1, B1

import { "-" as Imp9, "*" as Imp10 } from "module";
export {A1 as "-", B1 as "*"};
export {A5 as "--", B5 as "**"} from "a.js";

