import assert from "assert";
import { exec } from "child_process";

describe("bin/terser", function() {
    var tersercmd = '"' + process.argv[0] + '" bin/terser';
    it("Should be able to filter comments correctly with `--comments false`", function(done) {
        var command = tersercmd + " test/input/issue-585/input.js --comments false";

        exec(command, function (err, stdout) {
            if (err) throw err;

            assert.strictEqual(stdout, "\n");
            done();
        });
    });
});
