"use strict";var __awaiter=this&&this.__awaiter||function(thisArg,_arguments,P,generator){function adopt(value){return value instanceof P?value:new P((function(resolve){resolve(value)}))}return new(P||(P=Promise))((function(resolve,reject){function fulfilled(value){try{step(generator.next(value))}catch(e){reject(e)}}function rejected(value){try{step(generator["throw"](value))}catch(e){reject(e)}}function step(result){result.done?resolve(result.value):adopt(result.value).then(fulfilled,rejected)}step((generator=generator.apply(thisArg,_arguments||[])).next())}))};function somePromiseFn(){return __awaiter(this,void 0,void 0,(function*(){let value="promise-value";return value}))}somePromiseFn().then((d=>console.info(d)));
//# sourceMappingURL=data:application/json;charset=utf-8;base64,eyJ2ZXJzaW9uIjozLCJuYW1lcyI6WyJzb21lUHJvbWlzZUZuIiwidmFsdWUiLCJ0aGVuIiwiZCIsImNvbnNvbGUiLCJpbmZvIl0sInNvdXJjZXMiOlsiaW5wdXQudHN4Il0sInNvdXJjZXNDb250ZW50IjpbImFzeW5jIGZ1bmN0aW9uIHNvbWVQcm9taXNlRm4oKTogUHJvbWlzZTxzdHJpbmc+IHtcbiAgbGV0IHZhbHVlID0gJ3Byb21pc2UtdmFsdWUnO1xuICByZXR1cm4gdmFsdWU7IFxufVxuXG5zb21lUHJvbWlzZUZuKCkudGhlbihkID0+IGNvbnNvbGUuaW5mbyhkKSk7Il0sIm1hcHBpbmdzIjoieWpCQUFBLFNBQWVBLGdCLGlEQUNiLElBQUlDLE1BQVEsZ0JBQ1osT0FBT0EsS0FDVCxHLENBRUFELGdCQUFnQkUsTUFBS0MsR0FBS0MsUUFBUUMsS0FBS0YifQ==
